##########################################################################################
##########################################################################################
##########################################################################################
##########################################################################################
##########################################################################################
##########################################################################################

#install.packages('dplyr')
library(dplyr)     # sane data manipulation
library(tidyr)     # sane data munging
library(ggplot2)   # nice plots
#library(ggfortify) # super-helpful for plotting non-"standard" stats objects

#identifying your working directory
getwd()  #confirm your working directory is accurate

library(readr)
library(datasets)
data(iris)
summary(iris)
df<-iris

dim(iris)

#spec(iris)
head(iris)

names(df) <- tolower(names(df))
library(dplyr)
virginica <- filter(df, species == "virginica")
head(virginica) # This dispalys the first six rows

sepalLength6 <- filter(df, species == "virginica", sepal.length > 6)
tail(sepalLength6) # compare this to head()

# select() the specified columns
selected <- select(df, sepal.length, sepal.width, petal.length)
# select all columns from sepal.length to petal.length
selected2 <- select(df, sepal.length:petal.length)
head(selected, 3)

identical(selected, selected2)

#mutate() Create new columns using this function

#create a new column that stores logical values for sepal.width greater than half of sepal.length
newCol <- mutate(df, greater.half = sepal.width > 0.5 * sepal.length)
tail(newCol)

# arrange()
newCol <- arrange(newCol, petal.width)
head(newCol)

arr.virg <- newCol %>% filter(species == "virginica") %>%
  arrange(sepal.width)
arr.virg[30:35,] # will show us rows 30 through 35 and all columns

plot(df)

plot(df$sepal.width, df$sepal.length)

# ?hist will give you details on more arguments
hist(df$sepal.width)

library(ggplot2)
#Scatter Plot
ggplot(iris, aes(Petal.Length, Petal.Width)) + geom_point(aes(color=Species), size=2)

#As we can see, ‘setosa’ is going to be clustered easier. Meanwhile, there is noise 
#between ‘versicolor’ and ‘virginica’ even when they look like perfectly clustered.
#Let’s run the model. ‘kmeans’ is installed in the base package from R, so we don’t have to install any package.
#In the ‘kmeans’ function, it is necessary to set center, which is the number of groups we want to cluster to. 
#In this case, we know this value will be 3 ..

set.seed(101)
irisCluster <- kmeans(x=iris[,1:4], center=3, nstart=20)
#x=data
#center=the number of components k (or, alternatively, or a set of 
#initial (distinct) cluster centres).
#The nstart option attempts multiple initial configurations and reports on 
#the best one. For example, adding nstart=25 will generate 25 initial 
#configurations. This approach is often recommended.

irisCluster
#We can compare the predicted clusters with the original data.
table(irisCluster$cluster, iris$Species)
ggplot(iris, aes(Petal.Length, Petal.Width)) + geom_point(aes(color=irisCluster$cluster), size=2, show.legend = FALSE) 
ggplot(iris, aes(Petal.Length, Sepal.Width )) + geom_point(aes(color=irisCluster$cluster), size=2, show.legend = FALSE)
ggplot(iris, aes(Petal.Length, Sepal.Length)) + geom_point(aes(color=irisCluster$cluster), size=2, show.legend = FALSE)
ggplot(iris, aes(Sepal.Width, Sepal.Length)) + geom_point(aes(color=irisCluster$cluster), size=2, show.legend = FALSE)

library(cluster)
clusplot(iris, irisCluster$cluster, color=T, shade=T, labels=0, lines=0)
###ATTENTION: clusplot Creates a bivariate plot visualizing a partition (clustering) of the data. 
#All observation are represented by points in the plot, using principal components 
#or multidimensional scaling. Around each cluster an ellipse is drawn.

#the Setosa cluster is perfectly explained, while virginica and versicolor
#have a little noise between their clusters.



#In practice, you will never have the labeled data. 
#If you would want to know the ‘exact’ number of centers, you should have built the ‘elbow’ method.

#‘Elbow’ method.
tot.withinss <- vector(mode="character", length=10)
for (i in 1:10){
  irisCluster <- kmeans(df[,1:4], center=i, nstart=20)
  tot.withinss[i] <- irisCluster$tot.withinss
}

plot(1:10, tot.withinss, type="b", pch=19)

#the between_ss/total_ss ratio tends to change slowly and remain less changing 
#as compared to other k’s.so for this data k=4 should be a good choice for number
#of clusters however k=5 also seems to be a potential candidate.

help(silhouette)
irisCluster <- kmeans(x=iris[,1:4], center=3, nstart=20)
s_i<-silhouette(irisCluster$cluster,dist(iris[,1:4],method = "euclidean"))
summary(s_i)

s.bar<-vector(mode="numeric", length=9)
for (i in 2:10){
  irisCluster <- kmeans(df[,1:4], center=i, nstart=20)
  s_i<-silhouette(irisCluster$cluster,dist(iris[,1:4],method = "euclidean")) 
  s.bar[i] <-mean(s_i[,3])
}
plot(1:10, s.bar, type="b", pch=19)
k.est<-which(s.bar==max(s.bar))
irisCluster <- kmeans(df[,1:4], center=k.est, nstart=20)
irisCluster$cluster
clusplot(iris, irisCluster$cluster, color=T, shade=T, labels=0, lines=0)

#################################################################
#### Model based clustering methods: Gaussian mixture models #### 
#################################################################

library(mclust)
mixture <- Mclust(iris[,1:4])

library(factoextra)
# BIC values used to choose the number of clusters.
fviz_mclust(mixture, "BIC", palette = "jco")
# Classification 
fviz_mclust(mixture, "classification", geom = "point",pointsize = 1.5, palette = "jco")
fviz_mclust(mixture, "uncertainty", palette = "jco")


##########################################################################################
##########################################################################################
##########################################################################################
##########################################################################################
##########################################################################################
##########################################################################################


help(silhouette)
mtcarsCluster <- kmeans(x=mtcars[,-c(8,9)], center=3, nstart=20)
mtcarsCluster
ggplot(mtcars[,-c(8,9)], aes(mpg, qsec)) + geom_point(aes(color=mtcarsCluster$cluster), size=2, show.legend = FALSE)
ggplot(mtcars[,-c(8,9)], aes(mpg, wt)) + geom_point(aes(color=mtcarsCluster$cluster), size=2, show.legend = FALSE)
#With clusplot we can visualize the data in a single plot. Better for clustering of 
#multivariate data!
clusplot(mtcars[,-c(8,9)], mtcarsCluster$cluster, color=T, shade=T, labels=0, lines=0)

tot.withinss <- vector(mode="character", length=10)
for (i in 1:10){
  mtcarsCluster <- kmeans(x=mtcars[,-c(8,9)], center=i, nstart=20)
  tot.withinss[i] <- mtcarsCluster$tot.withinss
  
}
plot(1:10, tot.withinss, type="b", pch=19)


s.bar<-vector(mode="numeric", length=9)
for (i in 2:10){
  mtcarsCluster <- kmeans(x=mtcars[,-c(8,9)], center=i, nstart=20)
  s_i<-silhouette(mtcarsCluster$cluster,dist(mtcars[,-c(8,9)],method = "euclidean"))
  s.bar[i] <-mean(s_i[,3])
}

plot(1:10, s.bar, type="b", pch=19)
k.est<-which(s.bar==max(s.bar))
mtcarsCluster <- kmeans(x=mtcars[,-c(8,9)], center=k.est, nstart=20)
mtcarsCluster
clusplot(mtcars[,-c(8,9)], mtcarsCluster$cluster, color=T, shade=T, labels=0, lines=0)

#### Hierarchical aggregation methods #### 
#We will start by testing a hierarchical aggregation method,
#in this case we do not need to decide the number of clusters a priori. 
#For this we will use the hclust function with the ward method, which seeks 
#to minimize the variance within each cluster, starting with clusters of only 1 element and joining them with 
#the lowest variance growth criterion (by the Lance-Williams algorithm).

#We will:
#Calulate the dissimilarity matrix
#Fit the hierarchical aggregation algorithm
#Look at the dendrogram and calculate other indeces for choosing the optimal number 
#of clusters.
#Compare the results with the k-means.

d<-daisy(mtcars[,-c(8,9)], metric = "euclidean") #calculate the dissimilarity matrix
mtcarsHC <- hclust(d=d, method="average")
#d=dissimilarity matrix
#method=the agglomeration method to be used: single linkage, complete linkage, 
#average linkage and others check: help(hclust)
#For this application, we use the average linkage
library("factoextra")
fviz_dend(mtcarsHC, k = NULL, # Cut in four groups
          cex = 0.5, # label size
          k_colors = c("#2E9FDF", "#00AFBB", "#E7B800"),
          color_labels_by_k = TRUE, # color labels by groups
          rect = TRUE # Add rectangle around groups
)

fviz_dend(mtcarsHC, k = 3, # Cut in four groups
          cex = 0.5, # label size
          k_colors = c("#2E9FDF", "#00AFBB", "#E7B800"),
          color_labels_by_k = TRUE, # color labels by groups
          rect = TRUE # Add rectangle around groups
)

fviz_dend(mtcarsHC, k = 2, # Cut in four groups
          cex = 0.5, # label size
          k_colors = c("#2E9FDF", "#00AFBB", "#E7B800"),
          color_labels_by_k = TRUE, # color labels by groups
          rect = TRUE # Add rectangle around groups
)

groups<-cutree(mtcarsHC, k = 2)
table(groups)
clusplot(mtcars[,-c(8,9)], groups, color=T, shade=T, labels=0, lines=0,main = "Hierarchical Clustering")
s_i<-silhouette(groups,dist(mtcars[,-c(8,9)],method = "euclidean"))
mean(s_i[,3])

clusplot(mtcars[,-c(8,9)], mtcarsCluster$cluster, color=T, shade=T, labels=0, lines=0,main = "K-means")
clusplot(mtcars[,-c(8,9)], groups, color=T, shade=T, labels=0, lines=0,main = "Hierarchical Clustering")
